/*  VER 215   TAB P   $Id: doit.c,v 1.18.2.5 2003/01/21 08:01:08 egil Exp $
 *
 *  an NNTP news exchange client
 *
 *  copyright 1996, 1997 Egil Kvaleberg, egil@kvaleberg.no
 *  the GNU General Public License applies
 *
 *  $Log: doit.c,v $
 *  Revision 1.18.2.5  2003/01/21 08:01:08  egil
 *  Renamed to PACKAGE_ style
 *
 *  Revision 1.18.2.4  2002/10/01 08:54:59  egil
 *  Changed sprintf to snprintf
 *
 *  Revision 1.18.2.3  2002/09/21 17:29:59  egil
 *  Fix authinfo with ihave
 *
 *  Revision 1.18.2.2  2002/01/29 07:21:31  egil
 *  Really
 *
 *  Revision 1.18.2.1  2002/01/29 07:19:22  egil
 *  Include strsignal()
 *
 *  Revision 1.18  1999/04/05 19:01:51  src
 *  Lock for scanlogs
 *
 *  Revision 1.17  1999/03/16 08:12:46  src
 *  Moved clean_active() to active.c
 *
 *  Revision 1.16  1999/03/16 08:03:30  src
 *  Renamed lock() to do_lock()
 *
 *  Revision 1.15  1999/03/11 07:30:00  src
 *  Implemented check for spool free space
 *
 *  Revision 1.14  1998/11/22 08:00:07  src
 *  Option --no-queue
 *
 *  Revision 1.13  1998/10/21 07:31:22  src
 *  Opened syslog at an an earlier stage
 *
 *  Revision 1.12  1998/09/11 16:37:43  src
 *  Lockfile for logfile of posted articles and for posted article folder.
 *
 *  Revision 1.11  1998/09/09 07:32:11  src
 *  Version 1.1
 *
 *  Revision 1.10  1998/09/03 03:51:35  src
 *  Fixed bad syntax messages
 *
 *  Revision 1.9  1998/09/02 06:50:30  src
 *  newsx version 1.0
 *
 *  Revision 1.8  1998/09/02 06:34:41  src
 *  Support @-syntax in newsfeeds, and support AUTHINFO GENERIC
 */

#include "common.h"
#include "proto.h"
#include "options.h"
#include "newsconfig.h"
#include "nntp.h" 
#include "news.h"

#ifdef HAVE_STRSIGNAL
#define _GNU_SOURCE /* ensure strsignal() definition */
#endif
#include <string.h>
#include <signal.h>

/*
 *  locals
 */
static void 
set_signals(void);
static RETSIGTYPE 
sig_ignore(int signo);

static int did_interrupt = 0;

#ifdef HAVE_STRSIGNAL
    char *strsignal(int sig);
#endif

/*
 *  do it
 *  return exit status
 */
int 
doit(void)
{
    int ret;
    time_t starttime,endtime;
    char *article_name = 0;

    progtitle("begin");
    set_signals();

    if (enquire_opt) {
	if (!history_lookup(enquire_opt)) {
	    printf("no\n");
	    return 12;
	}
	printf("yes\n");
	return 0;
    }

    /*
     *  wait in case scanlogs is running 
     *  NOTE: this probably only makes sense if scanlogs is patched to
     *        read newsx logs
     */
    if (inn_opt && scanlogs_lock) {
	int d = 0;
	char lockpath[PATH_MAX];

	progtitle("check scanlogs running");
	build_filename(lockpath,LOCKS,"/","LOCK.scanlogs",NULL);
	while (check_lock(lockpath)) {
	    if (d==0) progtitle("await scanlogs lock");
	    if ((d += LOCKDELTA) > SCANLOGTIME) {
		log_msg(L_ERR,"scanlog lock \"%s\" timed out", lockpath);
		return 13;
	    }
	    sleep(LOCKDELTA);
	}
    }

    if (!no_host_lock) {
	/* 
	 *  lock other newsxes accessing same host out first
	 */
	char lockname[PATH_MAX];

	progtitle("locking host access");
	build_filename(lockname,LOCK_PREFIX,hostname,NULL,NULL);
	do_lock(NULL,lockname,noqueue);
    }

    time(&starttime);
    log_msg(L_DEBUG,"%s %s %s", PACKAGE_NAME, PACKAGE_VERSION, text_time(starttime));
    log_msg(L_DEBUG,"server: %s, spool: %s", hostname, spoolname);

    /* load the active file before we connect to the server */
    if (!nopull_opt || group_newlist || group_desc) {
	load_sys();
	load_active(0);
    }

    /* see if there are any articles batched up */
    if (!nopost_opt && !(article_name = despool_line())) {
	log_msg(L_DEBUG,"outgoing spool is empty");
	if (nopull_opt) {
	    /* don't bother */
	    return 0;
	}
    }

    /* set up the connection to the server */
    if (!noaction_opt) switch (ret = open_server(hostname,hostport)) {
    case -1 :
	return 3;
    case OK_CANPOST:
	++post_allowed;
	log_msg(L_DEBUG,"OK, can post");
	break;
    case OK_NOPOST:
	log_msg(L_DEBUG,"OK, but can't post");
	break;
    default:
	log_msg(L_ERR,"can't talk to \"%s\": got response code %d", hostname, ret);
	return 4;
    }

    /* start the timer now */
    time(&starttime);

    /*
     * we might need to do MODE READER before AUTHINFO or afterwards.
     * readbeforeauth_opt switches which way round we do things.
     * The fact that we don't do MODE READER until after IHAVE
     * transfers complicates things a little... -- PMM
     */

    /* if authinfo details supplied, then use 'em */
    if (!readbeforeauth_opt && ai_username)
       do_authinfo(ai_username,ai_password);
	
    /* switch INN to nnrpd instead of innd if needed */
    if (!ihave_opt && mode_reader_opt > 0) {
      do_mode_reader();
      /* If we haven't already authenticated, do that now */
      if (readbeforeauth_opt && ai_username)
          do_authinfo(ai_username,ai_password);
    }

    /* get the active newgroup list from the server */
    if (group_list || group_newlist) {
	get_list();
	if (!nopull_opt || group_desc) {
	    /* the active list may have been updated by now... */
	    load_active(1);
	}
    }

    /* get the newgroup descriptions from the server */
    if (group_desc || group_alldesc) {
	get_desc(); 
    }

    /* now put the actual articles */
    if (article_name && !zap_opt)
	despool(article_name);

    /* might need to do the switch here */
    if (ihave_opt && mode_reader_opt >= 0) {
	do_mode_reader();
	/* If we postponed authentication until after MODE READER,
	 * better do it now.
	 */
	if (readbeforeauth_opt && ai_username)
	    do_authinfo(ai_username,ai_password);
    }

    /* and pull any new articles */
    if (!nopull_opt) {
	/*
	 *  check spool free space first
	 */
	if (minfree) {
	    long kbyte, inode;

	    /* BUG: what about cfg_incoming? */
	    log_msg(L_DEBUG,"check spool free space");
	    if (!freespace(cfg_spool, &kbyte, &inode)) {
		log_msg(errno ? L_ERRno:L_ERR,
			"can't determine free %s space",cfg_spool);
	    } else {
		/* BUG: what about inodes? */
		if (kbyte < minfree) {
		    log_msg(L_ERR,"spool free space is %ldk too low",
							minfree-kbyte);
		    goto no_fetch;
		}
		/*
		 *  NOTE: assume avg. 2k per article
		 */
		if (inode < minfree/2) {
		    log_msg(L_ERR,"spool free inodes is %ldk too low",
							minfree/2 - inode);
		}
	    }
	}

	log_msg(L_DEBUG,"fetching news");
	if (!pull(spoolname)) {
	    /*
	     *  socket timed out, or we were interrupted.
	     *  since this may have been caused by an impatient user
	     *  interrupting, we try to clean up after us by closing
	     *  the server connection as good as we can.
	     */
	    if (!noaction_opt)
		close_server(1);
	    return 9;
	}
    }
    no_fetch:

    /* time for goodbye */
    if (!noaction_opt)
	if (!close_server(0)) return 9;

    time(&endtime); 

    statistics(starttime,endtime);

    /* lock will be removed in main() */

    /* reached so far: everything is all right */
    return 0;
}

/*
 *  some don't have this, so give them a surrogate
 */
#ifndef HAVE_STRSIGNAL
char *
strsignal(int sig)
{
    static char buf[10];
    snprintf(buf,sizeof(buf),"%d",sig);
    return buf;
}
#endif

/*
 *  common for the next two functions...
 */
void
do_cleanup(char *title, int exitcode)
{
    static int again = 0;

    if (++again >= 10) {
	/* this definitely should not happen... */
	_exit(exitcode);
    }

    progtitle(title);

    /* clean up fetching and spooling */
    pull_cleanup(1);

    /* remove any locks */
    unlock_exit(exitcode);
}

/*
 *  exit program 
 *  save as much as we can, then clean up locks etc
 */
void
exit_cleanup(int exitcode)
{
    do_cleanup(did_interrupt ? "exit via interrupt" : "exit",exitcode);
}

/*
 *  signal handler to report signal in log and possibly
 *  submit remaining batch to news and dump uncollected message ids.
 */
static RETSIGTYPE 
sig_interrupt(int signo)
{
    /* pretty primitive, but it works of sort */
    extern int alarm_active;

    ++did_interrupt;
    if (alarm_active) {
	signal(signo, sig_interrupt); /* allow more interrupts */
	progtitle("interrupt -> alarm");
	log_msg(L_ERR,"received signal while alarm active: %s",
						strsignal(signo));
	/* alarm enabled, so treat it as such */
	sig_alrm(signo);
	return;
    }
    signal(signo, sig_ignore); /* ignore further interrupts of the same kind */
    log_msg(L_ERR,"received signal: %s", strsignal(signo));

    do_cleanup("interrupt",1);
}

/*
 *  signal handler to ignore signal
 */
static RETSIGTYPE 
sig_ignore(int signo)
{
    signal(SIGUSR1, sig_ignore);
    log_msg(L_DEBUGMORE,"received signal: %s", strsignal(signo));
}

/*
 *  set up signal handler to catch appropriate signals.
 */
static void 
set_signals(void)
{
    /* these signals all causes us to terminate gracefully: */
    signal(SIGHUP,  sig_interrupt);
    signal(SIGINT,  sig_interrupt);
    signal(SIGQUIT, sig_interrupt);
    signal(SIGTERM, sig_interrupt);

    /* this signal is just to check that we are alive: */
    signal(SIGUSR1, sig_ignore);
}

